@@ -11,6 +11,7 @@
 #   WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 #   See the License for the specific language governing permissions and
 #   limitations under the License.
+import warnings
 
 import numpy as np
 import pytensor
@@ -45,3 +46,10 @@ def strict_float32():
 def seeded_test():
     # TODO: use this instead of SeededTest
     np.random.seed(42)
+
+
+@pytest.fixture
+def fail_on_warning():
+    with warnings.catch_warnings():
+        warnings.simplefilter("error")
+        yield
